using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.Common;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;

using VaultAgent;
using VaultCmdLineClient;
using XmlViewerEnhanced;

namespace VaultAgent
{
    public partial class FrmAgent : Form
    {
       private string[]           _raw_args;
       private Args               cmdlineargs = null;
       private XMLOutputWriter     xml = null;
       private VaultCmdLineClient.VaultCmdLineClient cmdlineclient = null;
       private XmlViewer myXmlViewer;
       private bool _bNoSocket = false;

       public FrmAgent(string[] args)
        {
            InitializeComponent();
            _raw_args = args;

        }
        
        private static ServerSocket server = null;
        private Object MkThreadsSequential = new Object();

        private void RunServer(object stateInfo)
        {
            // fire it up
            if (server == null)
            {
                server = new ServerSocket();
                server.OnLog = delegate(NetworkStream ns, string cInput)
                {
                    Logregel("log : " + cInput);
                };
                server.OnError = delegate(NetworkStream ns, string cInput)
                {
                    Logregel("error: " + cInput);
                };
                server.OnDataRecieved = delegate(NetworkStream ns, string cInput)
                {
                    

                    try
                    {
                        string cOutput;
                        lock (MkThreadsSequential)
                        {
                            Logregel("in: " + cInput);
                            processBatchCommands(cInput, out cOutput);
                        }

                        if (ns != null)
                        {
                            // Buffer for writing data
                            byte[] bytes = Encoding.ASCII.GetBytes(cOutput);
                            ns.Write(bytes, 0, bytes.Length);

                        }
                    }
                    catch (Exception ex) { 
                        Logregel("error: " + ex.Message.Replace("\r","\r\n"));
                    }

                };

                int iPort;

                if (Int32.TryParse(cmdlineargs.SocketPort, out iPort))
                {
                   server.Listen(iPort);
                }
                else Logregel("error: " + "Portnumber is invalid; " + cmdlineargs.SocketPort);
            }
        }

        private void Logregel(string data)
        {
            SetText(DateTime.Now + " " + data + "\r\n");
        }

        private delegate void SetTextCallback(string text);

        private void SetText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (textBoxLog.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetText);
                Invoke(d, new object[] { text });
            }
            else
            {
                string s = textBoxLog.Text;
                textBoxLog.Text = text + s;
            }
        }

        private void stopListining()
        {
            if (server != null)
                server.StopListening();

        }

        private void FrmMain_Shown(object sender, EventArgs e)
        {
            string ret = "\r\n";

            if (_raw_args.Length == 0)
            {
                textBoxResults.Text = "VaultAgent needs command line parameters,"
                    + " similar to vault.exe in batch mode."
                    + ret + ret + "VaultAgent can process typed in batch commands, as well as incoming batch "
                    + "commands on a port of your choosing."
                    + ret + ret + "Example: VaultAgent.exe socket 12340 -user XXX -password XXX -server XXX -repository XXX"
                    + ret + ret + "Give option 'socket 0' for skipping the creation of the server-socket."
                    + ret + "Type 'vault.exe help' or 'vault.exe help batch' for more help on vault (batch) commands."
                    + ret + ret + "Please restart this program with the correct command-line parameters.";

                btnGo.Enabled = false;
                textBoxCommands.Enabled = false;

            }

            else
            {
                int iPort = 0;

                textBoxResults.Text = "CmdLineClient.CmdMain is executing.";

                // Process events to show Text
                Application.DoEvents();

                makeVaultCmdLineClient(_raw_args);

                textBoxResults.Text = cmdlineclient.LastOutputString;

                if (!Int32.TryParse(cmdlineargs.SocketPort, out iPort))
                    iPort = 0;

                if (iPort != 0)
                    this.Text += ":" + cmdlineargs.SocketPort;

                this.Text += " - " + cmdlineargs.User
                          +  " @ " + cmdlineargs.Repository;

                if (iPort != 0)
                    ThreadPool.QueueUserWorkItem(RunServer);
                else {
                    splitContainer1.SuspendLayout();
                    textBoxLog.Hide();
                    textBoxResults.Hide();
                    splitContainer2.Hide();
                    splitContainer2.Panel2.Controls.Remove(textBoxResults);
                    splitContainer1.Panel2.Controls.Add(textBoxResults);
                    splitContainer1.Panel2.Controls.Remove(splitContainer2);
                    splitContainer1.Panel2.Controls.Add(textBoxResults);
                    textBoxResults.Show();
                    _bNoSocket = true;
                    splitContainer1.ResumeLayout(true);
                }

                toolStripMenuItem1.Checked = true;
                toolStripMenuItem1_Click(toolStripMenuItem1, new EventArgs());

            }

        }

        private string convertEOL(string cInput)
        {
            return cInput.Replace("\n", "\r\n");
        }

        private void makeVaultCmdLineClient(string[] args)
        {
            // create the args.
            cmdlineargs = new Args(args);

            // create the output writer - based on the OUT option
            xml = new XMLOutputWriter(cmdlineargs.Out);

            // create the cmd line client object
            cmdlineclient = new VaultCmdLineClient.VaultCmdLineClient(cmdlineargs, xml);

            cmdlineclient.CmdLineProcess( cmdlineargs, xml);

        }

        private void btnGo_Click(object sender, EventArgs e)
        {
            if (myXmlViewer != null) myXmlViewer.Clear();
            textBoxResults.Text = "Processing...";

            Application.DoEvents();

            this.Cursor = Cursors.WaitCursor;
            btnGo.Enabled = false;

            string cOutput;
            processBatchCommands(textBoxCommands.Text, out cOutput );
            textBoxResults.Text = cOutput;
            toolStripMenuItem1_Click(toolStripMenuItem1, new EventArgs());
            btnGo.Enabled = true;
            this.Cursor = Cursors.Default;
        }

        public void processBatchCommands(string cInput, out string cOutput)
        {
            // parse and process the batch commands

            char[] split = new char[1] { '\n'};
            bool bTryOnceAgain = true;

            string[] cmdLine = cInput.Split(split, StringSplitOptions.RemoveEmptyEntries);
            for (int i = 0; i < cmdLine.GetLength(0); i++) cmdLine[i] = cmdLine[i].Trim();

            cOutput = "";

            lock (cmdlineargs) foreach (string _cmdLine in cmdLine)
            {
                if (_cmdLine != "" && !_cmdLine.StartsWith(";"))
                {
                    Args socket_args = cmdlineargs.CreateSocketCmdArgs();
                    socket_args.CmdRaw = _cmdLine;

                    try
                    {
                        this.UseWaitCursor = true;
                        Application.DoEvents();

                        socket_args.ParseBatchCommand(_cmdLine);

                        processBatchCmd(ref socket_args, ref cOutput);

                    }
                    catch (System.Web.Services.Protocols.SoapException sex)
                    {
                        processBatchCmdRetry(sex, ref bTryOnceAgain, ref socket_args, ref cOutput);
                    }
                    catch (Exception ex)
                    {
                        cOutput = cOutput + "\r\n" 
                            + ex.GetType().ToString() + "\r\n" 
                            + ex.Message.Replace("\r","\r\n") + "\r\n" 
                            + ex.StackTrace;
                        cmdlineclient.Logout(true);
                        Logregel("error processbatch: " + ex.Message.Replace("\r", "\r\n"));
                    }
                    finally
                    {
                        this.UseWaitCursor = false;
                        Application.DoEvents();
                    }
                }
            }
        }

        private void processBatchCmd(ref Args socket_args, ref string cOutput)
        {
            if (socket_args.Cmd != Command.NONE) lock (socket_args)
            {
                bool bOK = cmdlineclient.ProcessSocketCommand(socket_args);

                cOutput = cOutput + convertEOL(cmdlineclient.LastOutputString);

                if (!bOK) cOutput = cOutput + "\r\n\r\nError while processing command.";
            }
            else
            {
                cOutput = cOutput + "The command could not be processed.";
            }
        
        }

        private void processBatchCmdRetry(System.Web.Services.Protocols.SoapException sex, ref bool bTryOnceAgain, ref Args socket_args, ref string cOutput)
        {
            // With a soap exception, we try one time to logout and login,
            // to see if the server is up and running again (previous 
            // socket batch command could be a while ago).
            // If then logged in, we can process the command normally.
            // If not, logout and abort.

            if (bTryOnceAgain)
            {
                bTryOnceAgain = false;
                cmdlineclient.Logout(true);
                cmdlineclient.Login();
                try
                {
                    processBatchCmd(ref socket_args, ref cOutput);
                }
                catch (Exception ex)
                {
                    cOutput = cOutput + "r\n"
                            + ex.GetType().ToString() + "\r\n"
                            + ex.Message.Replace("\r", "\r\n") + "\r\n"
                            + ex.StackTrace;

                    cmdlineclient.Logout(true);
                }

            }
            else
            {
                cOutput = cOutput + "\n" + sex.Message + "\n" + sex.StackTrace;
                cmdlineclient.Logout(true);
            }
        }


        private void FrmAgent_FormClosed(object sender, FormClosedEventArgs e)
        {
            stopListining();
            if (cmdlineclient != null) cmdlineclient.Logout(true);

        }

        private void FrmAgent_Load(object sender, EventArgs e)
        {

            System.Reflection.Assembly thisAssembly = System.Reflection.Assembly.GetExecutingAssembly();
            Version thisVersion = thisAssembly.GetName().Version;

            textBoxCommands.Text =
                "; --- Welcome to VaultAgent " + thisVersion.ToString() + " ---" +
                "\r\n\r\n";

                //+
                //"history $/ -begindate 23-10-2006 " +
                //"-outfields date,name,type,version,user,actionstring," +
                //"miscinfo1,miscinfo2,fullpath,comment,affectedfiles";

                //"folderdiff $/ont $/prod -echocmd -syncscript" +
                //"\r\nlistfolder $/test -outfields fullpath,version,moddate," + 
                //"shares,lastuser,lastaction,lastcomment -echocmd";

            //"folderdiff $/A $/B -echocmd";
            //"listfolder $/ont/s/rp/comn/comn.p -outfields name,lastuser";
            //"help\r\nhelp listfolder\r\n; Comment starts with \";\", press \"Go\" to commit all commands.";

        }

        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem m = (ToolStripMenuItem)sender;
            if (m.Checked)
            {
                if (myXmlViewer == null) {
                    SuspendLayout();
                    myXmlViewer = new XmlViewerEnhanced.XmlViewer();

                    myXmlViewer.CloseRequest += new EventHandler(myXmlViewer_CloseRequest);

                    myXmlViewer.Dock = System.Windows.Forms.DockStyle.Fill;
                    myXmlViewer.Location = new System.Drawing.Point(0, 0);
                    myXmlViewer.Name = "myXmlViewer";
                    myXmlViewer.Size = new System.Drawing.Size(617, 344);
                    myXmlViewer.TabIndex = 0;
                    myXmlViewer.Enabled = true;
                    myXmlViewer.ContextMenu = textBoxResults.ContextMenu;
                    if (_bNoSocket)
                    {
                        splitContainer1.Panel2.ContextMenu = textBoxResults.ContextMenu;
                        splitContainer1.Panel2.Controls.Add(this.myXmlViewer);
                    }
                    else
                    {
                        // SplitContainer2 is in panel2 of SplitCOntainer1, but is
                        // only shown in server-socket mode
                        splitContainer2.Panel2.ContextMenu = textBoxResults.ContextMenu;
                        splitContainer2.Panel2.Controls.Add(this.myXmlViewer);
                    }

                    textBoxResults.Visible = false;
                    ResumeLayout();
                }

                System.Xml.XmlDocument myXml = new System.Xml.XmlDocument();

                try
                {
                    myXml.LoadXml("<viewxml>\r\n" + textBoxResults.Text + "\r\n</viewxml>");
                }
                catch {
                    textBoxResults.Text = "No xml:" + "\r\n\r\n" + textBoxResults.Text;
                    myXmlViewer.Dispose();
                    myXmlViewer = null;
                    textBoxResults.Visible = true;
                }

                if (myXmlViewer != null) 
                    this.myXmlViewer.LoadDomXml(myXml);


            }
            else {
                if ((myXmlViewer != null) && 
                    (!myXmlViewer.Disposing)) 
                    myXmlViewer.Dispose();
                myXmlViewer = null;
                textBoxResults.Visible = true;
            }

        }

        private void myXmlViewer_CloseRequest(object sender, EventArgs e)
        {
            toolStripMenuItem1.Checked = false;
            toolStripMenuItem1_Click(toolStripMenuItem1, new EventArgs());
        }
    }
}

;